<?php
// $Id: help.admin.inc,v 1.4 2008/06/24 17:35:34 Gurpartap Exp $

/**
 * @file
 * Admin page callbacks for the help module.
 */

/**
 * Menu callback; prints a page listing a glossary of Drupal terminology.
 */
function help_index_page($module = '') {
  $output = '';
/*  // Add CSS
  drupal_add_css(drupal_get_path('module', 'help') . '/help.css', 'module', 'all', FALSE);
  $output .= '<h2>' . t('Help topics') . '</h2><p>' . t('Help is available on the following items:') . '</p>' . help_links_as_list();
*/
  $topics = help_get_topics();

  $breadcrumb = array();
  if ($module) {
    if (empty($topics[$module])) {
      return drupal_not_found();
    }

    help_get_topic_hierarchy($topics);
    $items = help_get_tree($topics, $topics[$module]['']['children'], array(), 0);

    $breadcrumb[] = help_l('Help', 'admin/help');

    drupal_set_title(t('@module help index', array('@module' => help_get_module_name($module))));
    $output .= theme('item_list', $items, NULL, 'ul', array('id' => 'toc'));
  }
  else {
    // Print a module index.
    $modules = array();
    $result = db_query("SELECT * FROM {system}");
    while ($info = db_fetch_object($result)) {
      $module_info = unserialize($info->info);
      $modules[$info->name] = $module_info['name'];
    }

    asort($modules);

    $items = array();
    foreach ($modules as $module => $module_name) {
      if (!empty($topics[$module])) {
        $items[] = help_l($module_name, "admin/help/$module");
      }
    }

    $output .= theme('item_list', $items);
  }

  if (!empty($_GET['popup'])) {
    $GLOBALS['devel_shutdown'] = FALSE; // Prevent devel module from spewing.
    drupal_set_breadcrumb(array_reverse($breadcrumb));
    print theme('help_popup', $output);
    return;
  }

  $breadcrumb = array_merge(drupal_get_breadcrumb(), array_reverse($breadcrumb));
  drupal_set_breadcrumb($breadcrumb);

  return $output;
}

/**
 * Menu callback; prints a page with help topic for a module.
 */
function help_topic_page($module, $topic = NULL) {
  $info = help_get_topic($module, $topic);
  if (isset($topic) && !$info) {
    return drupal_not_found();
  }

  $popup = !empty($_GET['popup']);

  drupal_set_title($info['title']);

  // Set up breadcrumb.
  $breadcrumb = array();

  $parent = $info;
  $pmodule = $module;

  // Loop checker.
  $checked = array();
  while (!empty($parent['parent'])) {
    if (strpos($parent['parent'], '%')) {
      list($pmodule, $ptopic) = explode('%', $parent['parent']);
    }
    else {
      $ptopic = $parent['parent'];
    }

    if (!empty($checked[$pmodule][$ptopic])) {
      break;
    }
    $checked[$pmodule][$ptopic] = TRUE;

    $parent = help_get_topic($pmodule, $ptopic);
    if (!$parent) {
      break;
    }

    $breadcrumb[] = help_l($parent['title'], "admin/help/$pmodule/$ptopic");
  }

  $breadcrumb[] = help_l(help_get_module_name($pmodule), "admin/help/$pmodule");
  $breadcrumb[] = help_l(t('Help'), "admin/help");

  $output = help_view_topic($module, $topic, $popup);
  if (empty($output)) {
    $output = t('Missing help topic.');
  }

  if ($popup) {
    $GLOBALS['devel_shutdown'] = FALSE; // Prevent devel module from spewing.
    drupal_set_breadcrumb(array_reverse($breadcrumb));
    print theme('help_popup', $output);
    return;
  }

  if (!$popup) {
    $breadcrumb[] = help_l('Administer', 'admin');
  }

  drupal_add_css(drupal_get_path('module', 'help') . '/help.css');
  $breadcrumb[] = l('Home', '');
  drupal_set_breadcrumb(array_reverse($breadcrumb));
  return $output;
}

/**
 * Load and render a help topic.
 */
function help_view_topic($module, $topic, $popup = FALSE) {
  $file = help_get_topic_filename($module, $topic);
  $info = help_get_topic($module, $topic);
  if ($file) {
    // @todo is this trusted output?
    $output = file_get_contents($file);
    // Make some exchanges. The strtr is because url() translates $ into %24
    // but we need to change it back for the regex replacement.

    // Run the line break filter if requested
    if (!empty($info['line break'])) {
      // Remove the header since it adds an extra <br /> to the filter.
      $output = preg_replace('/^<!--[^\n]*-->\n/', '', $output);

      $output = _filter_autop($output);
    }

    // Change 'topic:' to the URL for another help topic.
    if ($popup) {
      $output = preg_replace('/href="topic:([^"]+)"/', 'href="' . strtr(url('admin/help/$1', array('query' => 'popup=true')), array('%24' => '$')) . '"', $output);
      $output = preg_replace('/src="topic:([^"]+)"/', 'src="' . strtr(url('admin/help/$1', array('query' => 'popup=true')), array('%24' => '$')) . '"', $output);
    }
    else {
      $output = preg_replace('/href="topic:([^"]+)"/', 'href="' . strtr(url('admin/help/$1'), array('%24' => '$')) . '"', $output);
      $output = preg_replace('/src="topic:([^"]+)"/', 'src="' . strtr(url('admin/help/$1'), array('%24' => '$')) . '"', $output);
    }

    global $base_path;

    // Change 'path:' to the URL to the base help directory.
    $output = preg_replace('/href="path:([^"]+)"/', 'href="' . $base_path . $info['path'] . '/$1"', $output);
    $output = preg_replace('/src="path:([^"]+)"/', 'src="' . $base_path . $info['path'] . '/$1"', $output);

    // Change 'base_url:' to the URL to the site.
    $output = preg_replace('/href="base_url:([^"]+)"/', 'href="' . strtr(url('$1'), array('%24' => '$')) . '"', $output);
    $output = preg_replace('/src="base_url:([^"]+)"/', 'src="' . strtr(url('$1'), array('%24' => '$')) . '"', $output);

    if (!empty($info['navigation'])) {
      $topics = help_get_topics();
      help_get_topic_hierarchy($topics);

      if (!empty($topics[$module]['']['children'])) {
        $tree = array($topic);
        if (!empty($topics[$module][$topic]['parent'])) {
          array_push($tree, $topics[$module][$topic]['parent']);
        }
        $items = help_get_tree($topics, $topics[$module]['']['children'], $tree);
        $output = theme('item_list', $items, NULL, 'ul', array('id' => 'toc-block')) . $output;
      }
      if (!empty($topics[$module][$topic]['children'])) {
        $items = help_get_tree($topics, $topics[$module][$topic]['children']);
        $output .= theme('item_list', $items, NULL, 'ol', array('id' => 'toc'));
      }

      list($parent_module, $parent_topic) = $topics[$module][$topic]['_parent'];
      $siblings = $topics[$parent_module][$parent_topic]['children'];
      uasort($siblings, 'help_uasort');
      $prev = $next = NULL;
      $found = FALSE;
      foreach ($siblings as $sibling) {
        list($sibling_module, $sibling_topic) = $sibling;
        if ($found) {
          $next = $sibling;
          break;
        }
        if ($sibling_module == $module && $sibling_topic == $topic) {
          $found = TRUE;
          continue;
        }
        $prev = $sibling;
      }

      if ($prev || $next) {
        $navigation = '<div class="help-navigation clear-block">';

        $navigation .= '<div class="help-previous">';
        if ($prev) {
          $navigation .= help_l('<< ' . $topics[$prev[0]][$prev[1]]['title'], "admin/help/$prev[0]/$prev[1]");
        }
        $navigation .= '</div>';

        $navigation .= '<div class="help-next">';
        if ($next) {
          $navigation .= help_l($topics[$next[0]][$next[1]]['title'] . ' >>', "admin/help/$next[0]/$next[1]");
        }
        $navigation .= '</div>';

        $navigation .= '</div>';

        $output .= $navigation;
      }
    }

    if (!empty($info['css'])) {
      drupal_add_css($info['path'] . '/' . $info['css']);
    }

    return '<div class="help-topic">' . $output . '</div>';
  }
}


/*function help_links_as_list() {
  $empty_arg = drupal_help_arg();
  $module_info = module_rebuild_cache();

  $modules = array();
  foreach (module_implements('help', TRUE) as $module) {
    if (module_invoke($module, 'help', "admin/help#$module", $empty_arg)) {
      $modules[$module] = $module_info[$module]->info['name'];
    }
  }
  asort($modules);

  // Output pretty four-column list
  $count = count($modules);
  $break = ceil($count / 4);
  $output = '<div class="clear-block"><div class="help-items"><ul>';
  $i = 0;
  foreach ($modules as $module => $name) {
    $output .= '<li>' . l($name, 'admin/help/' . $module) . '</li>';
    if (($i + 1) % $break == 0 && ($i + 1) != $count) {
      $output .= '</ul></div><div class="help-items' . ($i + 1 == $break * 3 ? ' help-items-last' : '') . '"><ul>';
    }
    $i++;
  }
  $output .= '</ul></div></div>';

  return $output;
}*/

/**
 * Menu callback; prints a page listing general help for a module.
 */
/*function help_topic_page($name) {
  $output = '';
  if (module_hook($name, 'help')) {
    $module = drupal_parse_info_file(drupal_get_path('module', $name) . '/' . $name . '.info');
    drupal_set_title($module['name']);

    $temp = module_invoke($name, 'help', "admin/help#$name", drupal_help_arg());
    if (empty($temp)) {
      $output .= t("No help is available for module %module.", array('%module' => $module['name']));
    }
    else {
      $output .= $temp;
    }

    // Only print list of administration pages if the module in question has
    // any such pages associated to it.
    $admin_tasks = system_get_module_admin_tasks($name);
    if (!empty($admin_tasks)) {
      ksort($admin_tasks);
      $output .= theme('item_list', $admin_tasks, t('@module administration pages', array('@module' => $module['name'])));
    }

  }
  return $output;
}*/

/**
 * Build a tree of help topics.
 */
function help_get_tree($topics, $topic_ids, $tree_parents = array(), $max_depth = -1, $depth = 0) {
  uasort($topic_ids, 'help_uasort');
  $items = array();
  foreach ($topic_ids as $info) {
    list($module, $topic) = $info;
    $item = help_l($topics[$module][$topic]['title'], "admin/help/$module/$topic");
    if (!empty($tree_parents) && $topic == end($tree_parents)) {
      $children = !empty($topics[$module][$topic]['children']) ? $topics[$module][$topic]['children'] : array();
      $item .= theme('item_list', help_get_tree($topics, $children, $tree_parents, $max_depth, $depth + 1));
    }
    else if (empty($tree_parents) && !empty($topics[$module][$topic]['children']) && ($max_depth == -1 || $depth < $max_depth)) {
      $item .= theme('item_list', help_get_tree($topics, $topics[$module][$topic]['children'], $tree_parents, $max_depth, $depth + 1));
    }

    $items[] = $item;
  }

  return $items;
}

/**
 * Build a hierarchy for a single module's topics.
 */
function help_get_topic_hierarchy(&$topics) {
  foreach ($topics as $module => $module_topics) {
    foreach ($module_topics as $topic => $info) {
      $parent_module = $module;
      // We have a blank topic that we don't want parented to
      // itself.
      if (!$topic) {
        continue;
      }

      if (empty($info['parent'])) {
        $parent = '';
      }
      else if (strpos($info['parent'], '%')) {
        list($parent, $parent_module) = explode($info['parent'], '%');
        if (empty($topics[$parent_module][$parent])) {
          // If it doesn't exist, top level.
          $parent = '';
        }
      }
      else {
        $parent = $info['parent'];
        if (empty($module_topics[$parent])) {
          // If it doesn't exist, top level.
          $parent = '';
        }
      }

      if (!isset($topics[$parent_module][$parent]['children'])) {
        $topics[$parent_module][$parent]['children'] = array();
      }
      $topics[$parent_module][$parent]['children'][] = array($module, $topic);
      $topics[$module][$topic]['_parent'] = array($parent_module, $parent);
    }
  }
}

/**
 * Get the information for a single help topic.
 */
function help_get_topic($module, $topic) {
  $topics = help_get_topics();
  if (!empty($topics[$module][$topic])) {
    return $topics[$module][$topic];
  }
}

/**
 * Search the system for all available help topics.
 */
function help_get_topics() {
  static $topics = NULL;
  if (!isset($topics)) {
    $topics = array();
    $help_path = drupal_get_path('module', 'help') . '/modules';
    foreach (module_list() as $module) {
      $module_path = drupal_get_path('module', $module);
      $info = array();
      if (file_exists("$module_path/help/$module.help.ini")) {
        $path = "$module_path/help";
        $info = parse_ini_file("./$module_path/help/$module.help.ini", TRUE);
      }
      if (!empty($info)) {
        // Get translated titles:
        global $language;
        if (file_exists("$module_path/translations/help/$language->language/$module.help.ini")) {
          $translation = parse_ini_file("$module_path/translations/help/$language->language/$module.help.ini", TRUE);
        }
        $settings = array();
        if (!empty($info['help settings'])) {
          $settings = $info['help settings'];
          unset($info['help settings']);
        }

        foreach ($info as $name => $topic) {
          // Each topic should have a name, a title, a file and of course the path.
          $file = !empty($topic['file']) ? $topic['file'] : $name;
          $topics[$module][$name] = array(
            'name' => $name,
            'title' => !empty($translation[$name]['title']) ? $translation[$name]['title'] : $topic['title'],
            'weight' => isset($topic['weight']) ? $topic['weight'] : 0,
            'parent' => isset($topic['parent']) ? $topic['parent'] : 0,
            'file' => $file . '.html', // require extension
            'path' => $path, // not in .ini file
            'line break' => isset($topic['line break']) ? $topic['line break'] : (isset($settings['line break']) ? $settings['line break'] : FALSE),
            'navigation' => isset($topic['navigation']) ? $topic['navigation'] : (isset($settings['navigation']) ? $settings['navigation'] : TRUE),
            'css' => isset($topic['css']) ? $topic['css'] : (isset($settings['css']) ? $settings['css'] : NULL),
          );
        }
      }
    }
  }
  return $topics;
}

/**
 * Sort + http://php.net/uasort
 */
function help_uasort($id_a, $id_b) {
  $topics = help_get_topics();

  list($module_a, $topic_a) = $id_a;
  $a = $topics[$module_a][$topic_a];

  list($module_b, $topic_b) = $id_b;
  $b = $topics[$module_b][$topic_b];

  $a_weight = isset($a['weight']) ? $a['weight'] : 0;
  $b_weight = isset($b['weight']) ? $b['weight'] : 0;
  if ($a_weight != $b_weight) {
    return ($a_weight < $b_weight) ? -1 : 1;
  }

  if ($a['title'] != $b['title']) {
    return ($a['title'] < $b['title']) ? -1 : 1;
  }
  return 0;
}

/**
 * Load and render a help topic.
 */
function help_get_topic_filename($module, $topic) {
  init_theme();
  global $language;

  $info = help_get_topic($module, $topic);
  if (empty($info)) {
    return;
  }

  // Search paths:
  $paths = array(
    path_to_theme() . '/help', // Allow theme override.
    drupal_get_path('module', $module) . "/translations/help/$language->language", // Translations.
    $info['path'], // In same directory as .inc file.
  );

  foreach ($paths as $path) {
    if (file_exists("./$path/$info[file]")) {
      return "./$path/$info[file]";
    }
  }
}

/**
 * Small helper function to get a module's proper name.
 */
function help_get_module_name($module) {
  $info = db_fetch_object(db_query("SELECT * FROM {system} WHERE name = '%s'", $module));
  $info = unserialize($info->info);
  return t($info['name']);
}

/**
 * Format a link but preserve popup identity.
 */
function help_l($text, $dest, $options = array()) {
  if (!empty($_GET['popup'])) {
    if (empty($options['query'])) {
      $options['query'] = array();
    }

    if (is_array($options['query'])) {
      $options['query'] += array('popup' => TRUE);
    }
    else {
      $options['query'] += '&popup=TRUE';
    }
  }

  return l($text, $dest, $options);
}

/**
 * Format a URL but preserve popup identity.
 */
function help_url($dest, $options = array()) {
  if (!empty($_GET['popup'])) {
    if (empty($options['query'])) {
      $options['query'] = array();
    }

    $options['query'] += array('popup' => TRUE);
  }

  return url($dest, $options);
}
